# Artifact Appendix
Paper title: **Topology-Based Reconstruction Prevention for Decentralised Learning**

Artifacts HotCRP Id: **#3**

Requested Badge: **Reproduced**


## Description
Simulates reconstruction attacks with varying parameters to show their feasibility, and measures performance of distributed averaging as a function of the network's girth.

### Security/Privacy Issues and Ethical Concerns
No issues or concerns. All code consists of numerical simulations on artificial datasets.


## Basic Requirements
### Hardware Requirements
High CPU compute power. More cores is better.

### Software Requirements
Code is written in MATLAB. This is a proprietary tool that requires a license. The code is not compatible with GNU Octave.

The following packages are required:
* MATLAB R2023a or newer. Tested on MATLAB R2024b, which is the recommended version.
* MATLAB Parallel Computing Toolbox
* MATLAB Statistics and Machine Learning Toolbox

You can download MATLAB at <https://mathworks.com/downloads/>. For more information on installing MATLAB and its toolboxes, see <https://mathworks.com/help/install/ug/install-products-with-internet-connection.html>.

### Estimated Time and Storage Consumption
| **Physical cores** | 8        | 16       | 32       |
|--------------------|----------|----------|----------|
| **Time**           | 28 hours | 15 hours | 11 hours |
| **Memory**         | 36 GB    | 42 GB    | 54 GB    |
| **Storage**        | 717 MB   | 717 MB   | 717 MB   |

> **Physical cores and logical cores**  
> Note the distinction between physical cores and logical cores. On a typical personal computer, you have two logical cores per physical core. If your computer says it has 16 logical cores, you likely have only 8 physical cores.

> **How many physical cores do I have?**  
> The number of physical cores can typically be found in your operating system's settings, task manager, or in some sort of "About" section. Search for `number of cores [windows / macos / kde / gnome / ...]` in your favourite search engine for more information.

> **Reducing execution time and memory usage**  
> You can choose to reduce execution time and memory usage at the cost of reduced output accuracy by appropriately adjusting the experiments' input parameters. Relevant instructions are provided in the "Experiments" section.


## Environment
### Accessibility
All code is available at <https://doi.org/10.4121/21572601.v2>. Clone the git repository linked on that page.

The distribution artefacts (ending in `.cache.zip`) are not required. Download them if you are interested in inspecting the experiment's outputs without reproducing them yourself. Relevant instructions are provided in the "Experiments" section.

### Set up the environment
1. (Optional: Inspect Only) Extract the distribution artefacts into the git repository. Ensure that `FeasibilitySim.mat` is in the same directory as `FeasibilitySim.m`, and that `cache/` is in the same directory as `PerformanceSim.m`.
2. Launch MATLAB.
3. Install the "Parallel Computing Toolbox" by MathWorks:
   1. At the top of the MATLAB window, click the "Home" tab, and click "Add-Ons". The "Add-On Explorer" will open in a new window.
   2. Search for "Parallel Computing Toolbox" and go to the toolbox's page.
   3. Confirm that the toolbox is published by MathWorks.
   4. Click "Install" and then click "Continue". MATLAB will close and an installer will open.
   5. Walk through the steps of the installer. After completion, the installer will close and MATLAB will open.
4. Similarly, install the "Statistics and Machine Learning Toolbox" by MathWorks.
5. Install the "Customizable Heat Maps" add-on by Ameya Deoras:
   1. Open the "Add-On Explorer" as described above.
   2. Search for "Customizable Heat Maps" and go to the add-on's page.
   3. Confirm that the add-on is published by Ameya Deoras.
   4. Click "Add" and then "Add to MATLAB" to install the add-on.
6. Similarly, install the "Fast Reduced Row Echelon Form" add-on by Armin Ataei.
7. (Optional) Similarly, install the "Progress bar (cli, gui, parfor)" add-on by HyunGwang Cho. If you choose not to install this add-on, no progress bar will be displayed.
8. Close the add-on explorer.
9. Ensure the path is set correctly:
   1. In the "Home" tab, to the left of the "Add-Ons" button, click "Set Path". The window "Set Path" will open.
   2. Scroll all the way to the bottom, and click on the path ending in "Customizable Heat Maps".
   3. Click the button "Move to Top" on the left.
   4. Click "Save", then "Close".

> **Important!**  
> You **must** repeat step 8 every time after (re)starting MATLAB. Otherwise, the Feasibility experiment will give an error when creating heatmaps.

### Testing the Environment
To test the environment, you will need the "Command Window" (typically in the bottom right) to execute a few commands.

1. Run the command `parpool(2)`. A parallel pool should be started using the 'Processes' profile. Wait for the pool to have started. Shut down the parallel pool again by running the command `delete(gcp('nocreate'))`. If you do not get any errors, the Parallel Computing Toolbox was installed correctly.
2. Run the command `randsample(10, 1)`. If you do not get any errors, the Statistics and Machine Learning Toolbox was installed correctly.
3. Run the command `heatmap([1 1 2], 1:3, 1)`.
   * If you get the error "Unrecognized function or variable 'heatmap'.", the add-on "Customizable Heat Maps" was not installed correctly.
   * If you get the error "Vector of x values contains duplicate values.", the path was not adjusted correctly. (See "Set up the environment".)
   * Otherwise, if a heatmap appears, the add-on "Customizable Heat Maps" was installed correctly.
4. Run the command `frref(eye(5))`. If you do not get any errors, the add-on "Fast Reduced Row Echelon Form" was installed correctly.


## Artifact Evaluation
### Main Results and Claims
#### Main Result 1: Feasibility
Reconstruction attacks are feasible for honest-but-curious adversaries. We sample all possible adversarial views for a random selection of networks, and find that the success rate of the attack is non-zero for a variety of parameters.

See Section 4.4 of the paper, including Figures 4, 5, and 6.

#### Main Result 2: Performance
Increasing the network's girth increases the number of rounds before convergence occurs in (i.e. decreases performance of) distributed averaging over Erdős-Rényi graphs.

See Section 5.5 of the paper, including Figure 8.

### Experiments
> **"array exceeds maximum array size preference" error**  
> If you receive this error at any point, your machine does not have sufficient memory available to store the necessary data structures. You can resolve this error in three ways:
> 1. Acquire more RAM, either by closing other programs, or by adding more RAM to your machine.
> 2. Allow MATLAB to temporarily store data on disk. To do so, find the "Home" tab at the top of MATLAB, click "Preferences", go to "Workspace", and disable the option "Limit the maximum array size to a percentage of RAM". Note, however, that doing so is likely to significantly slow down the experiments, possibly by several orders of magnitude.

#### Experiment 1: FeasibilitySim
* **Expected result:** Exact replicas of Figures 4, 5, and 6.
* **Claim:** See Main Result 1.
* **Requirements:**

  | **Physical cores** | 8        | 16       | 32       | 48       |
  |--------------------|----------|----------|----------|----------|
  | **Time**           | 20 hours | 12 hours | 8 hours  | 8 hours  |
  | **Memory**         | 36 GB    | 42 GB    | 54 GB    | 69 GB    |
  | **Storage**        | 150 MB   | 150 MB   | 150 MB   | 150 MB   |

1. Set up the environment as described earlier.
2. In MATLAB, navigate to the directory `FeasibilitySim/`.
3. Open the file `FeasibilitySim.m`.
4. Cached results are loaded if and only if they are present. Therefore:
    * (Default) To reproduce the results yourself, ensure that the file `FeasibilitySim/FeasibilitySim.mat` does not exist.
    * (Optional) To analyse the generated results without reproducing them yourself, make sure to include the file `FeasibilitySim/FeasibilitySim.mat`. This file should be extracted from `FeasibilitySim.cache.zip`, available at https://doi.org/10.4121/21572601.v2.
5. (Optional) To limit the number of physical cores used in the experiment, find `parallel_num_workers = Inf;` and change `Inf` to the desired value. A value of `Inf` indicates that all physical cores on your machine should be used.
6. (Optional) To reduce execution time and memory usage at the cost of accuracy, find `graphs_per_config = 1000;` and change `1000` to a lower value.
7. Run `FeasibilitySim.m`: With `FeasibilitySim.m` opened in the editor, click the "Editor" tab at the top, and click "Run".
8. Keep track of progress in the "Command Window". The experiment is complete when `# Done` is written to output.

Note that the created figures are stored in the `figs/` directory.

#### Experiment 2: PerformanceSim
* **Expected result:** Exact replica of Figure 8.
* **Claim:** See Main Result 2.
* **Requirements:**

  | **Physical cores** | 8        | 16       | 32       |
  |--------------------|----------|----------|----------|
  | **Time**           | 8 hours  | 3 hours  | 3 hours  |
  | **Memory**         | 27 GB    | 33 GB    | 46 GB    |
  | **Storage**        | 566 MB   | 566 MB   | 566 MB   |

1. Set up the environment as described earlier.
2. In MATLAB, navigate to the directory `PerformanceSim/`.
3. Cached results are loaded if and only if they are present. Therefore:
   * (Default) To reproduce the results yourself, ensure that the directory `PerformanceSim/cache/` is empty or does not exist.
   * (Optional) To analyse the generated results without reproducing them yourself, make sure to include the file starting with `PerformanceSim/cache/lab-v14-`. This file should be extracted from `PerformanceSim.cache.zip`, available at https://doi.org/10.4121/21572601.v2.
4. Open the file `PerformanceSim.m`.
5. (Optional) To limit the number of physical cores used in the experiment, find `parallel_max_workers = Inf` and change `Inf` to the desired value. A value of `Inf` indicates that all physical cores on your machine should be used.
6. (Optional) To reduce execution time and memory usage at the cost of accuracy, find `repeat_count = 1000` and change `1000` to a lower value.
7. Run `PerformanceSim.m`: With `PerformanceSim.m` opened in the editor, click the "Editor" tab at the top, and click "Run".
8. Keep track of progress in the "Command Window". The experiment is complete when `Done creating plots.` is written to output.

The output of each individual step is stored in the `cache/` directory. You can safely stop and resume `PerformanceSim` at any point without losing much progress.

The created figures are stored in the `figs/` directory.


## Limitations
None.


## Notes on Reusability
The code is documented, well-structured, and extensible. The code could easily be changed to consider different types of graphs, stretching algorithms, distributed protocols, and metrics.
